/*  double/dbck.c	check consistency of DouBle file  20 Dec 1993
 *
 *  Copyright (C) 1994 Jean-Marc Verbavatz  <jmv@receptor.mgh.harvard.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <linux/double.h>

struct dble_device db;
extern struct db_header dbh;

main(argc, argv)
int argc;
char **argv;
{
int	i;
u_long	n;
u_long	empty, total, blocks;
float	factor;

if(argc != 2 || !db_open(&db, argv[1])) usage(argv[0]);
/* Could check data from header here (I'll see to it later) */

total = blocks = empty = 0;
for(n = 0; n<db.oblocks; n++) {
	i = get_block(&db, n);
	if(i < 0) fprintf(stderr, "Error %d in block %ld\n", i, n);
	else if(i > 0) total += i, blocks++;
	else empty++;
}
printf("DouBle device version %d.%d\n", dbh.version>>8, dbh.version&0xff);
if(total > 0) {
	factor = (float)db.ratio/((float)total/blocks);
	total += db.addr_blocks;
	printf("Average compression factor (excluding headers) = %2.1f\n", factor);
	printf("Approximately %.0fK still available\n", factor*db.isize*(db.iblocks-total)/1024);
} else {
	factor = 0;
	printf("Empty\n");
	total += db.addr_blocks;
}
printf("%ld/%ld (%2d%%) blocks in use, %ld/%ld (%2d%%) physical space used.\n",
	blocks, db.oblocks, (int)(100*blocks/db.oblocks), total, db.iblocks,
	(int)(100*total/db.iblocks));
printf("%ld/%ld (%2d%%) empty blocks, %ld/%ld (%2d%%) physical space free.\n\n",
	empty, db.oblocks, (int)(100*empty/db.oblocks), db.iblocks-total, db.iblocks,
	(int)(100*(db.iblocks-total)/db.iblocks));
db_close();
if(factor > 0) {
	printf("Conclusion: ");
	if(db.iblocks > total) {
		factor = empty*db.ratio/factor/(db.iblocks-total);
		if(factor < 0.75)
			printf("Compression ratio better than expected (you're safe) !\n\n");
		else if(factor > 1.3)
			printf("Poor compression ratio; could run out of space (beware) !\n\n");
		else printf("So far, the compression ratio looks OK to me.\n\n");
	}
	else printf("Be careful; this thing is FULL !!\n\n");
}
}

usage(s)
char *s;
{
printf("usage: %s device_file\n", s);
printf("\tcheck consistency of \"DouBle\" device/file\n");
exit(1);
}
